import os
from typing import Optional
from autobahn.asyncio.component import Component, run
from autobahn.wamp.interfaces import ISession
from autobahn.wamp.types import PublishOptions
from autobahn.wamp.request import Publication

from reswarm.AutobahnConnection import getSerialNumber, create_application_component


class Reswarm:
    """Convenience class for easy to use message publishing to the RE platform.

    Example:

        rw = Reswarm()

        async def main():
            while True:
                publication = await rw.publish("test.publish.pw", 1, "two", 3, foo="bar")
                print(publication)
                await asyncio.sleep(3)


        if __name__ == "__main__":
            asyncio.get_event_loop().create_task(main())
            rw.run()
    """

    def __init__(self, serial_number: str = None) -> None:
        """Creates Reswarm Instance

        Args:
            serial_number (str, optional): serial_number of device.
            Defaults to None, in which case the environment variable DEVICE_SERIAL_NUMBER is used.
        """
        self._serial_number = getSerialNumber(serial_number)
        self._component = create_application_component(serial_number)
        self._session: ISession = None

        @self._component.on_join
        def onJoin(session, details):
            print("component joined")
            self._session = session

        @self._component.on_disconnect
        def onDisconnect(*args, **kwargs):
            print("component disconnected")
            self._session = None

        @self._component.on_leave
        def onLeave(*args, **kwargs):
            print("component left")
            self._session = None

    @property
    def component(self) -> Component:
        """The Autobahn Component

        Returns:
            Component
        """
        return self._component

    @property
    def session(self) -> Optional[ISession]:
        """The Autobahn Session

        Returns:
            Optional[ISession]
        """
        return self._session

    async def publish(self, topic: str, *args, **kwargs) -> Optional[Publication]:
        """Publishes to the RE Platform Message Router

        Args:
            topic (str): The URI of the topic to publish to, e.g. "com.myapp.mytopic1"

        Returns:
            Optional[Publication]: Object representing a publication
            (feedback from publishing an event when doing an acknowledged publish)
        """
        device_name = os.environ.get("DEVICE_NAME")

        extra = {
            "DEVICE_SERIAL_NUMBER": self._serial_number,
            "options": PublishOptions(acknowledge=True),
        }

        if device_name:
            extra["DEVICE_NAME"] = device_name

        if self._session is not None:
            pub = await self._session.publish(topic, *args, **kwargs, **extra)
            return pub
        else:
            print("cannot publish, not connected")

    async def publish_to_table(
        self, tablename: str, *args, **kwargs
    ) -> Optional[Publication]:
        """Publishes Data to a Table in the RE Platform

        Args:
            topic (str): The URI of the topic to publish to, e.g. "com.myapp.mytopic1"

        Returns:
            Optional[Publication]: Object representing a publication
            (feedback from publishing an event when doing an acknowledged publish)
        """

        if not tablename:
            raise Exception("Tablename must not be None or empty string!")

        swarm_key = os.environ.get("SWARM_KEY")
        app_key = os.environ.get("APP_KEY")

        if swarm_key is None:
            raise Exception("Envrionment variable SWARM_KEY not set!")

        if app_key is None:
            raise Exception("Environment variable APP_KEY not set!")

        topic = f"{swarm_key}.{app_key}.{tablename}"

        pub = await self.publish(topic, *args, **kwargs)
        return pub

    def run(self):
        """Runs the Component in the asyncio event loop."""
        run([self._component])



import os
from typing import Tuple
from autobahn.asyncio.component import Component
from autobahn.asyncio.wamp import ApplicationSession, ApplicationRunner
from autobahn.wamp import auth


CB_REALM = "userapps"

DATAPODS_WS_URI = "wss://cbw.datapods.io/ws-ua-usr"
STUDIO_WS_URI = "wss://cbw.record-evolution.com/ws-ua-usr"
LOCALHOST_WS_URI = "ws://localhost:8080/ws-ua-usr"

socketURIMap = {
    "https://studio.datapods.io": DATAPODS_WS_URI,
    "https://studio.record-evolution.com": STUDIO_WS_URI,
    "http://localhost:8085": LOCALHOST_WS_URI,
}


def getWebSocketURI():
    reswarm_url = os.environ.get("RESWARM_URL")
    if not reswarm_url:
        return STUDIO_WS_URI
    return socketURIMap.get(reswarm_url)


def getSerialNumber(serial_number: str = None) -> str:
    if serial_number is None:
        s_num = os.environ.get("DEVICE_SERIAL_NUMBER")
        if s_num is None:
            raise Exception("ENV Variable 'DEVICE_SERIAL_NUMBER' is not set!")
    else:
        s_num = serial_number
    return s_num


class AppSession(ApplicationSession):
    serial_number: str = None

    def onConnect(self):
        if self.serial_number is None:
            raise Exception("serial_number missing on AppSession")

        self.join(CB_REALM, ["wampcra"], self.serial_number)

    def onChallenge(self, challenge):
        if challenge.method == "wampcra":
            if self.serial_number is None:
                raise Exception("serial_number missing on AppSession")

            signature = auth.compute_wcs(
                self.serial_number, challenge.extra["challenge"]
            )
            return signature

        raise Exception("Invalid authmethod {}".format(challenge.method))


def create_application_session(
    serial_number: str = None,
) -> Tuple[ApplicationSession, ApplicationRunner]:
    """Creates an Autobahn ApplicationSession and ApplicationRunner, which connects to the RE Platform

    Args:
        serial_number (str, optional): serial_number of device.
        Defaults to None, in which case the environment variable DEVICE_SERIAL_NUMBER is used.

    Returns:
        Tuple[ApplicationSession, ApplicationRunner]
    """
    AppSession.serial_number = getSerialNumber(serial_number)

    runner = ApplicationRunner(
        url=getWebSocketURI(),
        realm=CB_REALM,
    )

    return AppSession, runner


def create_application_component(serial_number: str = None) -> Component:
    """Creates an Autobahn Component, which connects to the RE Platform

    Args:
        serial_number (str, optional): serial_number of device.
        Defaults to None, in which case the environment variable DEVICE_SERIAL_NUMBER is used.

    Returns:
        Component
    """
    appSession, _ = create_application_session(serial_number)

    return Component(
        transports=getWebSocketURI(),
        realm=CB_REALM,
        session_factory=appSession,
    )



from reswarm.AutobahnConnection import (
    create_application_component,
    create_application_session,
)

from reswarm.reswarm import Reswarm

__all__ = ["Reswarm", "create_application_component", "create_application_session"]

__version__ = "0.0.15"



from setuptools import setup, find_packages

requirements = []
with open("requirements.txt", "r") as fh:
    for line in fh:
        requirements.append(line.strip())

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="reswarm",
    version="0.0.15",
    description="Aids users in publishing data to a Record Evolution Datapod",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/RecordEvolution/reswarm-python",
    author="Record Evolution GmbH",
    author_email="marko.petzold@record-evolution.de",
    packages=find_packages(),
    license="MIT",
    python_requires=">=3.6",
    install_requires=requirements,
    classifiers=[],
)
